# from plone.app.dexterity.behaviors.metadata import ICategorization
from plone.app.multilingual.interfaces import LANGUAGE_INDEPENDENT
from plone.base.interfaces import ILanguage
from zope.interface import implementer


# Patch for hiding 'language' field from the edit form
# ICategorization['language'].readonly = True


@implementer(ILanguage)
class Language:
    def __init__(self, context):
        self.context = context

    def get_language(self):
        return self.context.language or LANGUAGE_INDEPENDENT

    def set_language(self, language):
        self.context.language = language
        self.context.reindexObject(idxs=["Language"])



from plone.app.multilingual.dx.interfaces import ILanguageIndependentField
from plone.schemaeditor.interfaces import IFieldEditorExtender
from plone.schemaeditor.interfaces import ISchemaContext
from zope import schema
from zope.component import adapter
from zope.component import provideAdapter
from zope.i18nmessageid import MessageFactory
from zope.interface import alsoProvides
from zope.interface import implementer
from zope.interface import Interface
from zope.interface import noLongerProvides
from zope.schema import interfaces
from zope.schema.interfaces import IField


PMF = MessageFactory("plone.app.multilingual")


class IFieldLanguageIndependent(Interface):
    languageindependent = schema.Bool(
        title=PMF("Language independent field"),
        description=PMF(
            "The field is going to be copied on all "
            "translations when you edit the content"
        ),
        required=False,
    )


@implementer(IFieldLanguageIndependent)
@adapter(interfaces.IField)
class FieldLanguageIndependentAdapter:
    def __init__(self, field):
        self.field = field

    def _read_languageindependent(self):
        return ILanguageIndependentField.providedBy(self.field)

    def _write_languageindependent(self, value):
        if value:
            alsoProvides(self.field, ILanguageIndependentField)
        else:
            noLongerProvides(self.field, ILanguageIndependentField)

    languageindependent = property(
        _read_languageindependent, _write_languageindependent
    )


# IFieldLanguageIndependent could be registered directly as a named adapter
# providing IFieldEditorExtender for ISchemaContext and IField. But we can
# also register a separate callable which returns the schema only if
# additional conditions pass:
@adapter(ISchemaContext, IField)
def get_li_schema(schema_context, field):
    fti = getattr(schema_context, "fti", None)
    lang_behavior = {
        "plone.app.multilingual.dx.interfaces.IDexterityTranslatable",
        "plone.translatable",
    }
    fti_behaviors = set(getattr(fti, "behaviors", []))
    if lang_behavior.intersection(fti_behaviors):
        return IFieldLanguageIndependent


# Register the callable which makes the field edit form know about the
# new schema:
provideAdapter(
    get_li_schema,
    provides=IFieldEditorExtender,
    name="plone.schemaeditor.languageindependent",
)


# And the adapter for getting/setting the value.
provideAdapter(FieldLanguageIndependentAdapter, provides=IFieldLanguageIndependent)



from plone.app.i18n.locales.browser.selector import LanguageSelector
from plone.app.multilingual.interfaces import ITG
from plone.app.multilingual.interfaces import NOTG
from plone.i18n.interfaces import ILanguageSchema
from plone.registry.interfaces import IRegistry
from zope.component import getUtility
from zope.component import queryAdapter
from zope.component.hooks import getSite
from ZTUtils import make_query


def addQuery(request, url, exclude=tuple(), **extras):
    """Adds the incoming GET query to the end of the url
    so that is propagated through the redirect hoops
    """
    formvariables = {}
    for k, v in request.form.items():
        if k not in exclude:
            formvariables[k] = v
    for k, v in extras.items():
        formvariables[k] = v
    try:
        if len(formvariables) > 0:
            url += "?" + make_query(formvariables)
    # Again, LinguaPlone did this try/except here so I'm keeping it.
    except UnicodeError:
        pass
    return url


def get_root_request(request):
    """If in a subrequest, go up to the root request and return it"""

    def parent_request(current_request):
        preq = current_request.get("PARENT_REQUEST", None)
        if preq:
            return parent_request(preq)
        return current_request

    return parent_request(request)


def getPostPath(context, request):
    """Finds the path to be added at the end of a context.

    This is useful because you might have a view or even something more long
    (form and widget traversing) at the very end of the absolute_url
    of a translated item.
    When you get the translated item absolute_url,
    you want to also have the eventual views etc ported over:
    this function does that.
    """
    # This is copied over from LinguaPlone
    # because there's a lot of knowledge embed in it.

    # We need to find the actual translatable content object. As an
    # optimization we assume it is within the last three segments.
    path = context.getPhysicalPath()
    path_info = get_root_request(request).get("PATH_INFO", "")
    match = [p for p in path[-3:] if p]
    current_path = [pi for pi in path_info.split("/") if pi]
    append_path = []
    stop = False
    while current_path and not stop:
        check = current_path.pop()
        if check == "VirtualHostRoot" or check.startswith("_vh_"):
            # Once we hit a VHM marker, we should stop
            break
        if check not in match:
            append_path.insert(0, check)
        else:
            stop = True
    if append_path:
        append_path.insert(0, "")
    return "/".join(append_path)


NOT_TRANSLATED_YET_TEMPLATE = "/not_translated_yet"


class LanguageSelectorViewlet(LanguageSelector):
    """Language selector for translatable content."""

    def languages(self):
        languages_info = super().languages()
        results = []
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ILanguageSchema, prefix="plone")
        translation_group = queryAdapter(self.context, ITG)
        if translation_group is None:
            translation_group = NOTG

        for lang_info in languages_info:
            # Avoid to modify the original language dict
            data = lang_info.copy()
            data["translated"] = True
            query_extras = {}
            if not settings.set_cookie_always:
                query_extras.update(
                    {
                        "set_language": data["code"],
                    }
                )
            post_path = getPostPath(self.context, self.request)
            if post_path:
                query_extras["post_path"] = post_path
            site = getSite()
            data["url"] = addQuery(
                self.request,
                site.absolute_url().rstrip("/")
                + "/@@multilingual-selector/%s/%s"
                % (translation_group, lang_info["code"]),
                **query_extras
            )
            results.append(data)
        return results



from plone.app.multilingual.events import ObjectTranslatedEvent
from plone.app.multilingual.events import ObjectWillBeTranslatedEvent
from plone.app.multilingual.events import TranslationRegisteredEvent
from plone.app.multilingual.events import TranslationRemovedEvent
from plone.app.multilingual.events import TranslationUpdatedEvent
from plone.app.multilingual.interfaces import IMutableTG
from plone.app.multilingual.interfaces import ITG
from plone.app.multilingual.interfaces import ITranslationFactory
from plone.app.multilingual.interfaces import ITranslationLocator
from plone.app.multilingual.interfaces import ITranslationManager
from plone.app.multilingual.interfaces import NOTG
from plone.app.multilingual.itg import addAttributeTG
from plone.app.uuid.utils import uuidToObject
from plone.base.interfaces import ILanguage
from plone.protect.interfaces import IDisableCSRFProtection
from plone.uuid.handlers import addAttributeUUID
from plone.uuid.interfaces import IUUID
from Products.CMFCore.utils import getToolByName
from zope.component.hooks import getSite
from zope.event import notify
from zope.globalrequest import getRequest
from zope.interface import alsoProvides
from zope.interface import implementer


@implementer(ITranslationManager)
class TranslationManager:
    def __init__(self, context):
        self.context = context
        if isinstance(context, str):
            self.tg = context
        else:
            self.tg = self.get_tg(context)
        site = getSite()
        self.pcatalog = getToolByName(site, "portal_catalog", None)

    def get_id(self, context):
        """If an object is created via portal factory we don't get a id, we
        have to wait till the object is really created.
        TODO: a better check if we are in the portal factory!
        """
        try:
            context_id = IUUID(context)
        # We must ensure that this case can't happen, any object translatable
        # will have an UUID (in any case we can be at the portal factory!)
        except KeyError:
            alsoProvides(getRequest(), IDisableCSRFProtection)
            addAttributeUUID(context, None)
            context.reindexObject(idxs=["UID"])
            context_id = IUUID(context)
        return context_id

    def get_tg(self, context):
        """If an object is created via portal factory we don't get a id, we
        have to wait till the object is really created.
        TODO: a better check if we are in the portal factory!
        """
        try:
            context_id = ITG(context)
        # We must ensure that this case can't happen, any object translatable
        # will have an TG (in any case we can be at the portal factory!)
        except TypeError:
            alsoProvides(getRequest(), IDisableCSRFProtection)
            addAttributeTG(context, None)
            context.reindexObject(idxs=["TranslationGroup"])
            context_id = ITG(context)
        return context_id

    def query_canonical(self):
        return self.tg

    def register_translation(self, language, content):
        """register a translation for an existing content"""
        if not language and language != "":
            raise KeyError("There is no target language")

        if type(content) == str:
            content = uuidToObject(content)

        # Check if exists and is not myself
        brains = self.pcatalog.unrestrictedSearchResults(
            TranslationGroup=self.tg, Language=language
        )
        if len(brains) > 0 and brains[0].UID != self.get_id(content):
            raise KeyError("Translation already exists")

        # register the new translation in the canonical
        IMutableTG(content).set(self.tg)
        content.reindexObject(idxs=("Language", "TranslationGroup"))
        notify(TranslationRegisteredEvent(self.context, content, language))

    def update(self):
        """Update the adapted item.

        If unregistered, register a Translation-Grouup (TG) for it and exit.

        Check that there aren't two translations on the same language
        This is to be used for changing the contexts language.
        If there is already an item in the same language,
        Remove the other items TG information and make the current adapted
        context the active language for the current TG.

        """
        language = ILanguage(self.context).get_language()
        brains = self.pcatalog.unrestrictedSearchResults(
            TranslationGroup=self.tg,
            Language=language,
        )
        if len(brains) == 0:
            # There is no translation within current TG for this language.
            self.register_translation(language, self.context)
            return
        # In case the language is already within the translated languages we are
        # going to orphan the old translation.
        brain = brains[0]
        content_id = self.get_id(self.context)
        if brain.UID == content_id:
            return

        # It is a different object -> remove the old one
        # We get the old uuid
        old_object = brain.getObject()
        IMutableTG(old_object).set(NOTG)
        old_object.reindexObject(idxs=("TranslationGroup",))
        notify(TranslationUpdatedEvent(self.context, old_object, language))

    def add_translation(self, language):
        """see interfaces"""
        if not language and language != "":
            raise KeyError("There is no target language")
        # event
        notify(ObjectWillBeTranslatedEvent(self.context, language))
        # create the translated object
        translation_factory = ITranslationFactory(self.context)
        translated_object = translation_factory(language)
        ILanguage(translated_object).set_language(language)
        # reindex the translated object
        translated_object.reindexObject()
        # register the new translation
        self.register_translation(language, translated_object)
        # event
        notify(ObjectTranslatedEvent(self.context, translated_object, language))

    def add_translation_delegated(self, language):
        """
        Creation is delegated to factory/++add++
        Lets return the url where we are going to create the translation
        """
        if not language and language != "":
            raise KeyError("There is no target language")
        # event
        notify(ObjectWillBeTranslatedEvent(self.context, language))
        # localize where we need to store the new object
        locator = ITranslationLocator(self.context)
        parent = locator(language)
        return parent

    def remove_translation(self, language):
        """see interfaces"""
        translation = self.get_translation(language)
        IMutableTG(translation).set(NOTG)
        translation.reindexObject(idxs=("TranslationGroup",))
        notify(TranslationRemovedEvent(self.context, translation, language))

    def get_translation(self, language):
        """see interfaces"""
        brains = self.pcatalog.unrestrictedSearchResults(
            TranslationGroup=self.tg, Language=language
        )
        if len(brains) != 1:
            return None
        return brains[0].getObject()

    def get_restricted_translation(self, language):
        """see interfaces"""
        brains = self.pcatalog.searchResults(
            TranslationGroup=self.tg, Language=language
        )
        if len(brains) != 1:
            return None
        return brains[0].getObject()

    def get_translations(self):
        """see interfaces"""
        translations = {}
        brains = self.pcatalog.unrestrictedSearchResults(
            TranslationGroup=self.tg,
        )
        for brain in brains:
            translations[brain.Language] = brain.getObject()
        return translations

    def get_restricted_translations(self):
        """see interfaces"""
        translations = {}
        brains = self.pcatalog.searchResults(TranslationGroup=self.tg)
        for brain in brains:
            translations[brain.Language] = brain.getObject()
        return translations

    def get_translated_languages(self):
        """see interfaces"""
        languages = []
        brains = self.pcatalog.unrestrictedSearchResults(TranslationGroup=self.tg)
        for brain in brains:
            if brain.Language not in languages:
                languages.append(brain.Language)
        return languages

    def has_translation(self, language):
        """see interfaces"""
        return language in self.get_translated_languages()



from logging import getLogger
from plone.app.multilingual.browser.setup import SetupMultilingualSite
from plone.app.multilingual.interfaces import ITranslatable
from plone.app.multilingual.itg import addAttributeTG
from plone.base.interfaces import INonInstallable
from Products.CMFPlone.utils import getToolByName
from zope.component.hooks import getSite
from zope.interface import implementer

import pkg_resources


try:
    pkg_resources.get_distribution("plone.app.contenttypes")
except pkg_resources.DistributionNotFound:
    HAS_PLONE_APP_CONTENTTYPES = False
else:
    HAS_PLONE_APP_CONTENTTYPES = True


@implementer(INonInstallable)
class HiddenProfiles:
    def getNonInstallableProfiles(self):
        """Prevents uninstall profile from showing up in the profile list
        when creating a Plone site.

        """
        return [
            "plone.app.multilingual:uninstall",
        ]


def init_pam(tool):
    """After installation run setup to create LRF and LIF."""
    setup_tool = SetupMultilingualSite()
    setup_tool.setupSite(getSite())


def step_default_various(context):
    if context.readDataFile("plone.app.multilingual_default.txt") is None:
        return
    portal = context.getSite()
    enable_translatable_behavior(portal)

    # Add the attribute to the site root now so plone.protect won't cry.
    addAttributeTG(portal, None)


def enable_translatable_behavior(portal):
    types_tool = portal.portal_types

    # Iterate through all Dexterity content type, except the site root
    all_ftis = types_tool.listTypeInfo()
    dx_ftis = (
        fti
        for fti in all_ftis
        if getattr(fti, "behaviors", False) and fti.getId() != "Plone Site"
    )
    for fti in dx_ftis:
        # Enable translatable behavior for all types
        if "plone.translatable" not in fti.behaviors:
            behaviors = list(fti.behaviors)
            behaviors.append("plone.translatable")
            fti._updateProperty("behaviors", tuple(behaviors))


def step_uninstall_various(context):
    if context.readDataFile("plone.app.multilingual_uninstall.txt") is None:
        return
    portal = context.getSite()
    disable_translatable_behavior(portal)
    disable_language_switcher(portal)


def disable_translatable_behavior(portal):
    """Remove p.a.multilingual behaviors from p.a.contenttypes types"""
    types_tool = portal.portal_types

    # Iterate through all Dexterity content type
    all_ftis = types_tool.listTypeInfo()
    dx_ftis = [x for x in all_ftis if getattr(x, "behaviors", False)]
    for fti in dx_ftis:
        # Disable translatable behavior from all types
        behaviors = [i for i in fti.behaviors if i != "plone.translatable"]
        fti._updateProperty("behaviors", behaviors)


def disable_language_switcher(portal):
    """Remove the use of language-switcher as default view for Plone Site"""
    tt = getToolByName(portal, "portal_types")
    site = tt["Plone Site"]
    site.view_methods = tuple(
        method for method in site.view_methods if method != "language-switcher"
    )
    if site.default_view == "language-switcher":
        site.default_view = "listing_view"

    log = getLogger("setuphandlers.disable_language_switcher")
    log.info("Language switcher disabled")
